import {Resp} from "../../../interface/response.ts";
import {Client, WSMessage, ReceivedMessage} from "./types.ts";
import {useLocalStore} from "../../../store/local.ts";

// 文件接收缓存
const fileReceiveCache: Record<string, {chunks: ArrayBuffer[], total: number, received: number, name: string, size: number, sender: string, timestamp: number}> = {};

export const handleFileChunk = (chunk: any, onFileReceived: (msg: import("./types").ReceivedMessage) => void) => {
    if (chunk.type === 'file') {
        // 文件元信息，初始化缓存
        fileReceiveCache[chunk.name + '_' + chunk.timestamp] = {
            chunks: [],
            total: chunk.totalChunks,
            received: 0,
            name: chunk.name,
            size: chunk.size,
            sender: chunk.sender,
            timestamp: chunk.timestamp
        };
    } else if (chunk.type === 'file-chunk') {
        const key = chunk.name + '_' + chunk.timestamp;
        const cache = fileReceiveCache[key];
        if (cache) {
            // 将 data 数组还原为 ArrayBuffer
            const uint8 = new Uint8Array(chunk.data);
            cache.chunks[chunk.chunkIndex] = uint8.buffer;
            cache.received++;
            // 全部收到
            if (cache.received === cache.total) {
                const blob = new Blob(cache.chunks);
                const url = URL.createObjectURL(blob);
                onFileReceived({
                    sender: cache.sender,
                    timestamp: cache.timestamp,
                    fileName: cache.name,
                    fileSize: cache.size,
                    fileBlobUrl: url,
                    isFile: true
                });
                delete fileReceiveCache[key];
            }
        }
    }
};

export interface RTCHandlerCallbacks {
    onChannelOpen: (type: 'sender' | 'receiver') => void;
    onMessageReceived: (message: ReceivedMessage) => void;
    onChannelClose: () => void;
}

export class RTCHandler {
    private rtcRef: React.MutableRefObject<RTCPeerConnection | null>;
    private callbacks: RTCHandlerCallbacks;

    constructor(rtcRef: React.MutableRefObject<RTCPeerConnection | null>, callbacks: RTCHandlerCallbacks) {
        this.rtcRef = rtcRef;
        this.callbacks = callbacks;
    }

    // 更新回调函数的方法
    updateCallbacks = (newCallbacks: RTCHandlerCallbacks) => {
        console.log('[D] Updating RTC handler callbacks');
        this.callbacks = newCallbacks;
    };

    setupDataChannel = async (ch: RTCDataChannel, type: 'sender' | 'receiver') => {
        console.log(`[D] Setting up data channel for type: ${type}`);
        ch.onopen = () => {
            console.log(`[D] 通道已打开！类型: ${type}`);
            console.log('[D] Calling onChannelOpen callback with type:', type);
            this.callbacks.onChannelOpen(type);
            useLocalStore.getState().setChannel(ch);
        };

        ch.onmessage = (e) => {
            console.log('[D] Received message:', e.data);
            try {
                const data = JSON.parse(e.data);
                if (data.type === 'message') {
                    // 处理文本消息
                    const message: ReceivedMessage = {
                        text: data.content,
                        timestamp: Date.now(),
                        sender: data.sender || '未知用户'
                    };
                    console.log('[D] Calling onMessageReceived callback with message:', message);
                    this.callbacks.onMessageReceived(message);
                } else if (data.type === 'file') {
                    // 处理文件消息
                    handleFileChunk(data, this.callbacks.onMessageReceived);
                }
            } catch (error) {
                // 如果不是JSON格式，当作普通文本处理
                const message: ReceivedMessage = {
                    text: e.data,
                    timestamp: Date.now(),
                    sender: '未知用户'
                };
                console.log('[D] Calling onMessageReceived callback with plain text message:', message);
                this.callbacks.onMessageReceived(message);
            }
        };

        ch.onclose = () => {
            console.log('[D] 通道关闭');
            console.log('[D] Calling onChannelClose callback');
            this.callbacks.onChannelClose();
            useLocalStore.getState().setChannel();
        };
    };

    handleBubbleClick = async (bubbleId: string, currentUserId: string) => {
        console.log(`[D] click id = ${bubbleId}`);
        const current_rtc = this.rtcRef.current;
        if (!current_rtc) return;

        current_rtc.onnegotiationneeded = async () => {
            const offer = await current_rtc.createOffer();
            console.log('[D] offer created', offer);
            await current_rtc.setLocalDescription(offer);
            const data = {
                id: bubbleId,
                from: currentUserId,
                offer: offer,
            };
            await fetch('/api/ulocal/offer', {
                method: 'POST',
                headers: {"Content-Type": "application/json"},
                body: JSON.stringify(data)
            });
        };

        current_rtc.onicecandidate = async (e) => {
            console.log('[D] on candidate ', e);
            await fetch('/api/ulocal/candidate', {
                method: 'POST',
                headers: {"Content-Type": "application/json"},
                body: JSON.stringify({candidate: e.candidate, id: currentUserId})
            });
        };

        const ch = current_rtc.createDataChannel('local', {ordered: true});
        await this.setupDataChannel(ch, 'sender');
    };

    handleWSEvent = async (e: MessageEvent) => {
        let current_id: string;
        let current_rtc: RTCPeerConnection | null;
        const msg = JSON.parse(e.data) as WSMessage;
        console.log('[D] ws event msg =', msg);

        switch (msg.type) {
            case "enter":
            case "leave":
                // 这些事件由父组件处理
                return;
            case "offer":
                const offer_data = msg.data as { id: string; from: string; offer: RTCSessionDescriptionInit };
                current_id = useLocalStore.getState().id;
                if (offer_data.id !== current_id) {
                    console.warn(`[W] wrong offer id, want = ${current_id}, got = ${offer_data.id}, data =`, offer_data);
                    return;
                }

                current_rtc = this.rtcRef.current;
                if (!current_rtc) {
                    console.warn('[W] rtc undefined');
                    return;
                }

                await current_rtc.setRemoteDescription(offer_data.offer);
                const answer = await current_rtc.createAnswer();
                if (!answer) {
                    console.log('[W] answer undefined');
                    return;
                }

                await current_rtc.setLocalDescription(answer);

                current_rtc.ondatachannel = (e) => {
                    this.setupDataChannel(e.channel, 'receiver');
                };

                await fetch('/api/ulocal/answer', {
                    method: 'POST',
                    headers: {"Content-Type": "application/json"},
                    body: JSON.stringify({id: offer_data.from, answer: answer})
                });
                return;

            case "answer":
                const answer_data = msg.data as { answer: RTCSessionDescriptionInit; id: string };
                current_id = useLocalStore.getState().id;
                if (answer_data.id !== current_id) {
                    console.warn(`[W] wrong answer id, want = ${current_id}, got = ${answer_data.id}, data =`, answer_data);
                }

                current_rtc = this.rtcRef.current;
                if (!current_rtc) {
                    console.warn('[W] rtc undefined');
                    return;
                }

                await current_rtc.setRemoteDescription(answer_data.answer);
                return;

            case "candidate":
                const candidate_data = msg.data as { candidate: RTCIceCandidateInit };
                current_rtc = this.rtcRef.current;
                if (!current_rtc) {
                    console.warn('[W] rtc undefined');
                    return;
                }
                if (!candidate_data.candidate) {
                    console.log('[W] candidate data null');
                    return;
                }
                await current_rtc.addIceCandidate(candidate_data.candidate);
                return;
        }
    };

    sendMessage = (msg: string, files: File[], senderName: string) => {
        const ch = useLocalStore.getState().channel;
        console.log('[D] ready to send:', msg, files, ch);
        const CHUNK_SIZE = 64 * 1024; // 64KB
        const BUFFERED_AMOUNT_THRESHOLD = 1 * 1024 * 1024; // 1MB

        if (ch && ch.readyState === 'open') {
            if (msg.trim()) {
                // 发送文本消息
                const messageData = {
                    type: 'message',
                    content: msg,
                    sender: senderName,
                    timestamp: Date.now()
                };
                ch.send(JSON.stringify(messageData));
            }

            if (files && files.length > 0) {
                files.forEach(file => {
                    const totalChunks = Math.ceil(file.size / CHUNK_SIZE);
                    const timestamp = Date.now();
                    // 先发送文件元信息
                    const fileData = {
                        type: 'file',
                        name: file.name,
                        size: file.size,
                        sender: senderName,
                        timestamp,
                        totalChunks
                    };
                    ch.send(JSON.stringify(fileData));

                    ch.bufferedAmountLowThreshold = BUFFERED_AMOUNT_THRESHOLD;

                    let offset = 0;
                    let chunkIndex = 0;
                    const reader = new FileReader();

                    function sendNextChunk() {
                        if (offset >= file.size) return;
                        if (ch && ch.bufferedAmount && ch.bufferedAmount > BUFFERED_AMOUNT_THRESHOLD) {
                            // 等待缓冲区变低
                            ch.addEventListener('bufferedamountlow', sendNextChunk, { once: true });
                            return;
                        }
                        const slice = file.slice(offset, offset + CHUNK_SIZE);
                        reader.onload = (e) => {
                            if (e.target?.result) {
                                ch?.send(JSON.stringify({
                                    type: 'file-chunk',
                                    name: file.name,
                                    timestamp,
                                    chunkIndex,
                                    totalChunks,
                                    data: Array.from(new Uint8Array(e.target.result as ArrayBuffer)),
                                }));
                                offset += CHUNK_SIZE;
                                chunkIndex++;
                                sendNextChunk();
                            }
                        };
                        reader.readAsArrayBuffer(slice);
                    }
                    sendNextChunk();
                });
            }
        }
    };
} 